package tests;

import algorithms.*;
import structures.basic.*;
import structures.list.*;
import structures.map.*;
import types.procedures.*;
import types.structures.*;
import procedures.factories.*;
import procedures.orderings.*;
import procedures.combinations.*;
import procedures.predicates.*;
import procedures.conversions.*;

public class Examples extends SimpleTestHarness{

	// Tests the methods in ListAlgorithms.
	public void testListAlgorithms( String title, IFactory factory ) {
		
		String t = title + ": ";
		Int i1 = new Int(1), i2 = new Int(2), i3 = new Int(3);
		Int i4 = new Int(4), i5 = new Int(5), i6 = new Int(6);
		
		IList empty = (IList)factory.create();
		IList l123 = empty.cons(i3).cons(i2).cons(i1);
		IList l246 = empty.cons(i6).cons(i4).cons(i2);
		IList l321 = empty.cons(i1).cons(i2).cons(i3);
		IList l13 = empty.cons(i3).cons(i1);
		
		this.test(t + "filter", l13,
				ListAlgorithms.filter(l123, new OddInt()));
		this.test(t + "map", l246,
				ListAlgorithms.map(l123, new DoubleInt()));
		this.test(t + "orMap true", true,
				ListAlgorithms.orMap(l123, new EvenInt()));
		this.test(t + "orMap false", false,
				ListAlgorithms.orMap(l246, new OddInt()));
		this.test(t + "andMap true", true,
				ListAlgorithms.andMap(l246, new EvenInt()));
		this.test(t + "andMap false", false,
				ListAlgorithms.andMap(l123, new OddInt()));
		this.test(t + "foldLeft", l321,
				(IList)ListAlgorithms.foldLeft(l123, empty, new ListCons()));
		this.test(t + "foldRight", l123,
				(IList)ListAlgorithms.foldRight(l123, empty, new ListCons()));
		
	}
	
	// Tests the methods in TreeAlgorithms
	public void testTreeAlgorithms( String title, IFactory factory ) {
		
		String t = title + ": ";
		
		Int i1 = new Int(1), i2 = new Int(2), i3 = new Int(3);
		Int i4 = new Int(4), i5 = new Int(5), i6 = new Int(6);
		
		ICompare order = new IntOrder();
		ISelect balanced = new AVLInvariant();
		
		ITree leaf = (ITree)factory.create();
		ITree t12 = leaf.hangLeft(i1, leaf.hangLeft(i2, leaf));
		ITree t23 = leaf.hangLeft(i2, leaf).hangLeft(i3, leaf);
		ITree t123 = leaf.hangLeft(i1, leaf).hangLeft(i2, leaf.hangLeft(i3, leaf));
		ITree t246 = leaf.hangLeft(i2, leaf).hangLeft(i4, leaf.hangLeft(i6, leaf));
		
		IList empty = new Empty();
		IList l123 = empty.cons(i3).cons(i2).cons(i1);
		IList l132 = empty.cons(i2).cons(i3).cons(i1);
		IList l213 = empty.cons(i3).cons(i1).cons(i2);
		IList l231 = empty.cons(i1).cons(i3).cons(i2);
		IList l312 = empty.cons(i2).cons(i1).cons(i3);
		IList l321 = empty.cons(i1).cons(i2).cons(i3);
		
		this.test(t + "insert big", t123,
				TreeAlgorithms.insertBalanced( t12, i3, order, balanced ) );
		this.test(t + "insert lil", t123,
				TreeAlgorithms.insertBalanced( t23, i1, order, balanced ) );

		this.test(t + "rotate left",
				leaf.hangLeft(i1, leaf).hangLeft(i2,leaf),
				TreeAlgorithms.rotateLeft(t12) );
		this.test(t + "rotate right",
				leaf.hangLeft(i2, leaf.hangLeft(i3, leaf)),
				TreeAlgorithms.rotateRight(t23) );
		
		this.test(t + "find top", i2, TreeAlgorithms.find(t123, i2, order));
		this.test(t + "find left", i1, TreeAlgorithms.find(t123, i1, order));
		this.test(t + "find right", i3, TreeAlgorithms.find(t123, i3, order));
		
		this.test(t + "map", t246, TreeAlgorithms.map(t123, new DoubleInt()));
		this.test(t + "orMap true", true,
				TreeAlgorithms.orMap(t123, new EvenInt()));
		this.test(t + "orMap false", false,
				TreeAlgorithms.orMap(t246, new OddInt()));
		this.test(t + "andMap true", true,
				TreeAlgorithms.andMap(t246, new EvenInt()));
		this.test(t + "andMap false", false,
				TreeAlgorithms.andMap(t123, new OddInt()));
		
		this.test(t + "foldRight", l123,
				(IList)TreeAlgorithms.foldRight(t123, empty, new ListCons()) );
		this.test(t + "foldLeft", l321,
				(IList)TreeAlgorithms.foldLeft(t123, empty, new ListCons()) );
		this.test(t + "foldRightPreOrder", l132,
				(IList)TreeAlgorithms.foldRightPreOrder(t123, empty, new ListCons()) );
		this.test(t + "foldLeftPreOrder", l312,
				(IList)TreeAlgorithms.foldLeftPreOrder(t123, empty, new ListCons()) );
		this.test(t + "foldRightPostOrder", l213,
				(IList)TreeAlgorithms.foldRightPostOrder(t123, empty, new ListCons()) );
		this.test(t + "foldLeftPostOrder", l231,
				(IList)TreeAlgorithms.foldLeftPostOrder(t123, empty, new ListCons()) );
		
	}
	
	// Tests the methods in IList.
	public void testList( String title, IFactory factory ) {

		String t = title + ": ";
		
		Int i1 = new Int(1), i2 = new Int(2), i3 = new Int(3);
		
		IList empty1 = (IList)factory.create();
		IList empty2 = (IList)factory.create();
		IList cons123a = empty1.cons(i3).cons(i2).cons(i1);
		IList cons123b = empty2.cons(i3).cons(i2).cons(i1);
		IList cons321 = empty2.cons(i1).cons(i2).cons(i3);
		
		this.test(t + "same Empty/Empty", true, empty1.same(empty2));
		this.test(t + "same Cons/Cons", true, cons123a.same(cons123b));
		this.test(t + "diff Cons/Empty", false, cons321.same(empty1));
		this.test(t + "diff Empty/Cons", false, empty2.same(cons321));
		this.test(t + "diff Cons/Cons", false, cons123a.same(cons321));
		
		this.test(t + "isEmpty Empty", true, empty1.isEmpty());
		this.test(t + "isEmpty Cons", false, cons123a.isEmpty());
		
		this.test(t + "getFirst 1", i1, (ISame)cons123a.getFirst());
		this.test(t + "getFirst 3", i3, (ISame)cons321.getFirst());
		
		this.test(t + "getRest 23", empty1.cons(i3).cons(i2),
		                            cons123b.getRest());
		this.test(t + "getRest 21", empty2.cons(i1).cons(i2),
		                            cons321.getRest());
	}
	
	// Tests the methods in ITree.
	public void testTree( String title, IFactory factory ) {
		
		String t = title + ": ";
		
		Int i1 = new Int(1), i2 = new Int(2), i3 = new Int(3);
		
		ITree leaf1 = (ITree)factory.create();
		ITree leaf2 = (ITree)factory.create();
		ITree node123a =
			leaf1.hangLeft(i1,leaf1)
			     .hangLeft(i2,leaf2.hangLeft(i3, leaf2));
		ITree node123b =
			leaf2.hangLeft(i1,leaf1)
				 .hangLeft(i2,leaf1.hangLeft(i3, leaf2));
		ITree node123c =
			leaf1.hangLeft(i1, leaf1.hangLeft(i2, leaf1))
			     .hangLeft(i3, leaf1);
		
		this.test(t + "same Leaf/Leaf", true, leaf1.same(leaf2));
		this.test(t + "same Node/Node", true, node123a.same(node123b));
		this.test(t + "diff Leaf/Node", false, leaf2.same(node123b));
		this.test(t + "diff Node/Leaf", false, node123b.same(leaf2));
		this.test(t + "diff Node/Node", false, node123a.same(node123c));
		
		this.test(t + "height Leaf", 0, leaf1.getHeight());
		this.test(t + "height Node 2", 2, node123a.getHeight());
		this.test(t + "height Node 3", 3, node123c.getHeight());
		
		this.test(t + "getData 2", i2, (ISame)node123a.getData());
		this.test(t + "getData 3", i3, (ISame)node123c.getData());
		
		this.test(t + "getLeft 1",
		         leaf1.hangLeft(i1, leaf1),
		         node123a.getLeft());
		this.test(t + "getLeft 12",
				leaf1.hangLeft(i1, leaf1.hangLeft(i2, leaf1)),
				node123c.getLeft());
		
		this.test(t + "getRight 3", leaf1.hangLeft(i3, leaf1),
		                        node123a.getRight());
		this.test(t + "getRight Leaf", leaf1, node123c.getRight());
		
	}
	
	// Tests the methods in IMap.
	public void testMap( String title, IFactory factory ) {
		
		String t = title + ": ";
		
		Int i1 = new Int(1), i2 = new Int(2), i3 = new Int(3);
		Str a = new Str("a"), b = new Str("b"), c = new Str("c"), d = new Str("d");
		Entry a1 = new Entry(i1,a);
		Entry b2 = new Entry(i2,b);
		Entry c2 = new Entry(i2,c);
		
		IMap blank = (IMap)factory.create();
		IMap Ma1b2 = blank.extend(i1,a).extend(i2,b);
		IMap Ma1c2 = Ma1b2.extend(i2,c);
		
		this.test(t + "isEmpty blank", true, blank.isEmpty());
		this.test(t + "isEmpty extend", false, Ma1b2.isEmpty());
		
		this.test(t + "contains blank", false, blank.contains(i3));
		this.test(t + "contains first", true, Ma1b2.contains(i1));
		this.test(t + "contains second", true, Ma1b2.contains(i2));
		this.test(t + "contains twice", true, Ma1c2.contains(i2));
		this.test(t + "contains absent", false, Ma1c2.contains(i3));
		
		this.test(t + "lookup a1 1st", a, Ma1b2.lookup(i1));
		this.test(t + "lookup b2 2nd", b, Ma1b2.lookup(i2));
		this.test(t + "lookup c2 3rd", c, Ma1c2.lookup(i2));
		
		this.test(t + "keys a1c2",
				Ma1c2.keys().same(new Cons(i1, new Cons(i2, new Empty()))) ||
				Ma1c2.keys().same(new Cons(i2, new Cons(i1, new Empty()))) );
		
		this.test(t + "values a1c2",
				Ma1c2.values().same(new Cons(a, new Cons(c, new Empty()))) ||
				Ma1c2.values().same(new Cons(c, new Cons(a, new Empty()))) );
		
		this.test(t + "entries a1c2",
				Ma1c2.entries().same(new Cons(a1, new Cons(c2, new Empty()))) ||
				Ma1c2.entries().same(new Cons(c2, new Cons(a1, new Empty()))) );
		
	}
	
	// Tests the methods in ISet.
	public void testSet( String title, IFactory factory ) {
	
		String t = title + ": ";
		
		Int i1 = new Int(1), i2 = new Int(2), i3 = new Int(3);
		
		ISet empty = (ISet)factory.create();
		ISet s12 = empty.insert(i1).insert(i2);
		ISet s121 = s12.insert(i1);
		
		this.test(t + "isEmpty blank", true, empty.isEmpty());
		this.test(t + "isEmpty insert", false, s12.isEmpty());
		
		this.test(t + "member empty", false, empty.member(i1));
		this.test(t + "member first", true, s12.member(i1));
		this.test(t + "member second", true, s12.member(i2));
		this.test(t + "member twice", true, s121.member(i1));
		this.test(t + "member absent", false, s12.member(i3));
		
		this.test(t + "elements s12",
				s121.elements().same(new Cons(i1,new Cons(i2,new Empty()))) ||
				s121.elements().same(new Cons(i2,new Cons(i1,new Empty()))) );
		
	}
	
	// Runs all tests and produces a report.
	public void runTests(){
		
		testList("AList", new ListFactory());
		testTree("ATree", new TreeFactory());
		testMap("SimpleMap", new SimpleMapFactory());
		testMap("HashMap", new OrderedMapFactory(new HashOrder()));
		testMap("IntMap", new OrderedMapFactory(new IntOrder()));
		testSet("SimpleSet", new SimpleSetFactory());
		testSet("HashSet", new OrderedSetFactory(new HashOrder()));
		testSet("IntSet", new OrderedSetFactory(new IntOrder()));
		
		testListAlgorithms("AList algo", new ListFactory());
		testTreeAlgorithms("ATree algo", new TreeFactory());
		
        // example of a full test report
        fullTestReport();
		
        // example of a report on failures
        testReport();
	}
	
	
}